using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Settings : PersistentSingletonMonoBehaviour<Settings>
{
    public event Action<bool> EventMusicChanged;
    public event Action<bool> EventSfxChanged;
    public event Action<bool> EventGamepadVibrationChanged;

    [SerializeField] private bool music;
    [SerializeField] private bool sfx;
    [SerializeField] private bool useGamepadVibration = true;

    private void Update()
    {
        if (Input.GetKeyDown(KeyCode.M))
        {
            Music = !Music;
        }
        if (Input.GetKeyDown(KeyCode.N))
        {
            Sfx = !Sfx;
        }
        if (Input.GetKeyDown(KeyCode.V))
        {
            UseGamepadVibration = !UseGamepadVibration;
        }
    }

    public bool Music
    {
        get { return music; }
        set
        {
            if (music == value)
                return;

            music = value;

            if (EventMusicChanged != null)
                EventMusicChanged(music);
        }
    }

    public bool Sfx
    {
        get { return sfx; }
        set
        {
            if (sfx == value)
                return;

            sfx = value;

            if (EventSfxChanged != null)
                EventSfxChanged(sfx);
        }
    }

    public bool UseGamepadVibration
    {
        get { return useGamepadVibration; }
        set
        {
            if (useGamepadVibration == value)
                return;

            useGamepadVibration = value;

            if (EventGamepadVibrationChanged != null)
                EventGamepadVibrationChanged(useGamepadVibration);
        }
    }
}
