﻿using UnityEngine;

public class PersistentSingletonMonoBehaviour<T> : MonoBehaviour where T : PersistentSingletonMonoBehaviour<T>
{
    private static T instance;
    protected bool InstanceExists { get { return instance != null; } }
    public static T Instance { get { return instance; } }

    protected virtual void Awake()
    {
        if (InstanceExists)
        {
            Instance.OnSceneSwitched();
            Destroy(gameObject);
            return;
        }

        instance = (T) this;
        DontDestroyOnLoad(gameObject);
    }

    protected virtual void OnSceneSwitched()
    {
    }
}
