﻿using UnityEngine;

public class ChooseGamepadInput : MonoBehaviour
{
    [SerializeField] private GamepadInput windowsNativePrefab;
    [SerializeField] private GamepadInput osxPrefab;
    [SerializeField] private GamepadInput defaultInputPrefab;
    [SerializeField] private GamepadInput debugKeyboardInputPrefab;

    private void Awake()
    {
        GamepadInput target = null;

        switch (Application.platform)
        {
            case RuntimePlatform.OSXEditor:
            case RuntimePlatform.OSXPlayer:
            case RuntimePlatform.OSXDashboardPlayer:
            case RuntimePlatform.OSXWebPlayer:
                target = osxPrefab;
                break;

            case RuntimePlatform.WindowsEditor:
            case RuntimePlatform.WindowsPlayer:
                target = windowsNativePrefab;
                break;
        }

        if (target == null)
            target = defaultInputPrefab;

        var gameManager = GameManager.Instance;

        var normalInputGameObject = (GameObject) Instantiate(target.gameObject);
        var normalInput = normalInputGameObject.GetComponent<GamepadInput>();

        if (Debug.isDebugBuild && (debugKeyboardInputPrefab != null))
        {
            var keyboardInputGameObject = (GameObject) Instantiate(debugKeyboardInputPrefab.gameObject);
            var keyboardInput = keyboardInputGameObject.GetComponent<GamepadInput>();

            var mixer = gameObject.AddComponent<GamepadInputMixer>();
            mixer.Initialize(normalInput, keyboardInput);

            gameManager.GamepadInput = mixer;
        }
        else
        {
            gameManager.GamepadInput = normalInput;
        }
    }
}
