﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class GamepadInputKeyboard : GamepadInput
{
    [Serializable]
    private class KeyboardAssignment
    {
        [SerializeField] private KeyCode buttonA;
        [SerializeField] private KeyCode buttonB;
        [SerializeField] private KeyCode buttonX;
        [SerializeField] private KeyCode buttonY;
        [SerializeField] private KeyCode buttonLeftBack;
        [SerializeField] private KeyCode buttonRightBack;
        [SerializeField] private KeyCode buttonBack;
        [SerializeField] private KeyCode buttonStart;
        [SerializeField] private string axisPrimaryX;
        [SerializeField] private string axisPrimaryY;
        [SerializeField] private string axisSecondaryX;
        [SerializeField] private string axisSecondaryY;
        [SerializeField] private string axisTrigger;

        public void Initialize()
        {
            buttonToIndex = new Dictionary<Button, KeyCode>();
            axisToIndex = new Dictionary<Axis, string>();

            buttonToIndex[Button.A] = buttonA;
            buttonToIndex[Button.B] = buttonB;
            buttonToIndex[Button.X] = buttonX;
            buttonToIndex[Button.Y] = buttonY;
            buttonToIndex[Button.Start] = buttonStart;
            buttonToIndex[Button.Back] = buttonBack;
            buttonToIndex[Button.LeftBack] = buttonLeftBack;
            buttonToIndex[Button.RightBack] = buttonRightBack;

            axisToIndex[Axis.PrimaryX] = axisPrimaryX;
            axisToIndex[Axis.PrimaryY] = axisPrimaryY;
            axisToIndex[Axis.SecondaryX] = axisSecondaryX;
            axisToIndex[Axis.SecondaryY] = axisSecondaryY;
            axisToIndex[Axis.Trigger] = axisTrigger;
        }

        public Dictionary<Button, KeyCode> buttonToIndex { get; private set; }
        public Dictionary<Axis, string> axisToIndex { get; private set; }
    }

    [SerializeField] private KeyboardAssignment[] keyboardAssignmentGamepads = new KeyboardAssignment[2];
    [SerializeField] private KeyboardAssignment keyboardAssignmentNoGamepad;

    private void Awake()
    {
        foreach (var keyboardAssignment in keyboardAssignmentGamepads)
        {
            keyboardAssignment.Initialize();
        }

        keyboardAssignmentNoGamepad.Initialize();
    }

    public override bool GetButton(int gamepadIndex, Button button)
    {
        return Input.GetKey(keyboardAssignmentGamepads[gamepadIndex - 1].buttonToIndex[button]);
    }

    public override bool GetButtonDown(int gamepadIndex, Button button)
    {
        return Input.GetKeyDown(keyboardAssignmentGamepads[gamepadIndex - 1].buttonToIndex[button]);
    }

    public override bool GetButtonUp(int gamepadIndex, Button button)
    {
        return Input.GetKeyUp(keyboardAssignmentGamepads[gamepadIndex - 1].buttonToIndex[button]);
    }

    public override bool GetButton(Button button)
    {
        return Input.GetKey(keyboardAssignmentNoGamepad.buttonToIndex[button]);
    }

    public override bool GetButtonDown(Button button)
    {
        return Input.GetKeyDown(keyboardAssignmentNoGamepad.buttonToIndex[button]);
    }

    public override bool GetButtonUp(Button button)
    {
        return Input.GetKey(keyboardAssignmentNoGamepad.buttonToIndex[button]);
    }

    public override float GetAxis(int gamepadIndex, Axis axis)
    {
        var axisName = keyboardAssignmentGamepads[gamepadIndex - 1].axisToIndex[axis];
        if (axisName.Length == 0)
            return 0;

        return Input.GetAxis(axisName);
    }

    public override void SetLowestVibrationThisFrame(int gamepadIndex, float left, float right)
    {
    }
}
