﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class GamepadInputUnityDefault : GamepadInput
{
    [SerializeField] private int buttonA;
    [SerializeField] private int buttonB;
    [SerializeField] private int buttonX;
    [SerializeField] private int buttonY;
    [SerializeField] private int buttonLeftBack;
    [SerializeField] private int buttonRightBack;
    [SerializeField] private int buttonBack;
    [SerializeField] private int buttonStart;
    [SerializeField] private int axisPrimaryX;
    [SerializeField] private int axisPrimaryY;
    [SerializeField] private bool axisPrimaryYReversed = true;
    [SerializeField] private int axisSecondaryX;
    [SerializeField] private int axisSecondaryY;
    [SerializeField] private bool axisSecordaryYReversed = true;
    [SerializeField] private int axisTrigger;
    [SerializeField] private bool axisTriggerReversed = false;
    [SerializeField] private int axisTriggerSecondary = -1;
    [SerializeField] private bool axisTriggerSecondaryReversed = false;

    private Dictionary<Button, int> buttonToIndex; 
    private Dictionary<Axis, int> axisToIndex;
    private Dictionary<Axis, bool> axisReversed;

    private void Awake()
    {
        buttonToIndex = new Dictionary<Button, int>();
        axisToIndex = new Dictionary<Axis, int>();
        axisReversed = new Dictionary<Axis, bool>();

        buttonToIndex[Button.A] = buttonA;
        buttonToIndex[Button.B] = buttonB;
        buttonToIndex[Button.X] = buttonX;
        buttonToIndex[Button.Y] = buttonY;
        buttonToIndex[Button.Start] = buttonStart;
        buttonToIndex[Button.Back] = buttonBack;
        buttonToIndex[Button.LeftBack] = buttonLeftBack;
        buttonToIndex[Button.RightBack] = buttonRightBack;

        axisToIndex[Axis.PrimaryX] = axisPrimaryX;
        axisToIndex[Axis.PrimaryY] = axisPrimaryY;
        axisToIndex[Axis.SecondaryX] = axisSecondaryX;
        axisToIndex[Axis.SecondaryY] = axisSecondaryY;
        axisToIndex[Axis.Trigger] = axisTrigger;

        axisReversed[Axis.PrimaryX] = false;
        axisReversed[Axis.PrimaryY] = axisPrimaryYReversed;
        axisReversed[Axis.SecondaryX] = false;
        axisReversed[Axis.SecondaryY] = axisSecordaryYReversed;
        axisReversed[Axis.Trigger] = axisTriggerReversed;
    }

    public override bool GetButton(int gamepadIndex, Button button)
    {
        return Input.GetKey("joystick " + gamepadIndex + " button " + buttonToIndex[button]);
    }

    public override bool GetButtonDown(int gamepadIndex, Button button)
    {
        return Input.GetKeyDown("joystick " + gamepadIndex + " button " + buttonToIndex[button]);
    }

    public override bool GetButtonUp(int gamepadIndex, Button button)
    {
        return Input.GetKeyUp("joystick " + gamepadIndex + " button " + buttonToIndex[button]);
    }

    public override bool GetButton(Button button)
    {
        return Input.GetKey("joystick button " + buttonToIndex[button]);
    }

    public override bool GetButtonDown(Button button)
    {
        return Input.GetKeyDown("joystick button " + buttonToIndex[button]);
    }

    public override bool GetButtonUp(Button button)
    {
        return Input.GetKey("joystick button " + buttonToIndex[button]);
    }

    public override float GetAxis(int gamepadIndex, Axis axis)
    {
        var value = Input.GetAxis("axis" + gamepadIndex + "_" + axisToIndex[axis]);

        if (axisReversed[axis])
            value *= -1;

        if ((axis == Axis.Trigger) && (axisTriggerSecondary > -1))
        {
            var valueSecond = Input.GetAxis("axis" + gamepadIndex + "_" + axisTriggerSecondary);
            if (axisTriggerSecondaryReversed)
                valueSecond *= -1;

            value = value + valueSecond;
        }

        return value;
    }

    public override void SetLowestVibrationThisFrame(int gamepadIndex, float left, float right)
    {
    }
}
